/***
*
*	Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
//			
//  hud.h
//
// class CHud declaration
//
// CHud handles the message, calculation, and drawing the HUD
//
#define RGB_YELLOWISH	0x00FFA000 // 255, 160, 0
#define RGB_REDISH		0x00FF1010 // 255, 160, 0
#define RGB_GREENISH	0x0000A000 // 0, 160, 0

#include "wrect.h"
#include "windows.h"
#include "mathlib.h"
#include "cdll_int.h"
#include "cdll_dll.h"
#include "render_api.h"
#include "enginecallback.h"
#include "randomrange.h"
#include "screenfade.h"
#include "hud/ammo.h"

typedef struct cvar_s	cvar_t;

#define DHN_DRAWZERO 		1
#define DHN_2DIGITS  		2
#define DHN_3DIGITS  		4
#define DHN_4DIGITS  		5
#define MIN_ALPHA	 		100
#define FADE_TIME			100

#define MAX_PLAYERS			64
#define MAX_TEAMS			64
#define MAX_TEAM_NAME		16

#define HUD_ACTIVE			1
#define HUD_INTERMISSION		2

#define MAX_PLAYER_NAME_LENGTH	32
#define MAX_MOTD_LENGTH		2048

//
//-----------------------------------------------------
//
class CHudBase
{
public:
	int m_iFlags; // active, moving,
	virtual ~CHudBase() {}
	virtual int Init( void ) { return 0; }
	virtual int VidInit( void ) { return 0; }
	virtual int Draw(float flTime) { return 0; }
	virtual void Think(void) {}
	virtual void Reset(void) {}
	virtual void InitHUDData( void ) {} // called every time a server is connected to

};

struct HUDLIST
{
	CHudBase	*p;
	HUDLIST	*pNext;
};

//
//-----------------------------------------------------
//

#include <assert.h>
#include <string.h>
#include <stdio.h>
#include "cl_entity.h"




#include "hud/health.h"

#include "hudChild.h"

//
class CHud
{
private:
	HUDLIST *m_pHudList;
	client_sprite_t *m_pSpriteList;
	int m_iSpriteCount;
	int m_iSpriteCountAllRes;
	float m_flMouseSensitivity;
	int m_iConcussionEffect;
	HSPRITE m_hsprLogo;
	int m_iLogo;
public:
	float m_flTime;	 // the current client time
	float m_fOldTime;	 // the time at which the HUD was last redrawn
	double m_flTimeDelta;// the difference between flTime and fOldTime
	Vector m_vecOrigin;
	Vector m_vecAngles;
	int m_iKeyBits;
	int m_iHideHUDDisplay;
	int m_iFOV;
	int m_Teamplay;
	int m_iRes;
	int MousePosX,MousePosY;
	cvar_t *m_pCvarDraw;
	cvar_t *default_fov;
	int m_iCameraMode;
	int m_iFontHeight;
	int DrawHudNumber( int x, int y, int iFlags, int iNumber, int r, int g, int b );
	int DrawHudString( int x, int y, int iMaxX, char *szString, int r, int g, int b );
	int DrawHudStringReverse( int xpos, int ypos, int iMinX, char *szString, int r, int g, int b );
	int DrawHudNumberString( int xpos, int ypos, int iMinX, int iNumber, int r, int g, int b );
	int GetNumWidth( int iNumber, int iFlags );
	int m_iHUDColor;
	int GuiButtonIndex;
	BOOL g_iVisibleMouse;
	BOOL freezeControls;
	HSPRITE m_hsprCursor;
private:
	// the memory for these arrays are allocated in the first call to CHud::VidInit()
	// when the hud.txt and associated sprites are loaded. freed in ~CHud()
	HSPRITE *m_rghSprites; // the sprites loaded from hud.txt
	wrect_t *m_rgrcRects;
	char *m_rgszSpriteNames;
public:
	HSPRITE GetSprite( int index ) { return (index < 0) ? 0 : m_rghSprites[index]; }
	wrect_t& GetSpriteRect( int index ) { return m_rgrcRects[index]; }
          int InitHUDMessages( void ); // init hud messages
	int GetSpriteIndex( const char *SpriteName ); // gets a sprite index, for use in the m_rghSprites[] array

	CHudAmmo		m_Ammo;
	CHudHealth	m_Health;
	CHudGeiger	m_Geiger;
	CHudBattery	m_Battery;
	CHudTrain		m_Train;
	CHudFlashlight	m_Flash;
	CHudMessage	m_Message;
	CHudScoreboard	m_Scoreboard;
	CHudStatusBar	m_StatusBar;
	CHudDeathNotice	m_DeathNotice;
	CHudSayText	m_SayText;
	CHudMenu		m_Menu;
	CHudAmmoSecondary	m_AmmoSecondary;
	CHudTextMessage	m_TextMessage;
	CHudStatusIcons	m_StatusIcons;
	CHudMOTD		m_MOTD;
	CHudBook        m_Book;
CHudPipbuck        m_pipBuck;
CHudCodeLock m_CodeLock;
	void Init( void );
	void VidInit( void );
	void Think(void);
	int Redraw( float flTime, int intermission );
	int UpdateClientData( client_data_t *cdata, float time );

	CHud() : m_iSpriteCount(0), m_pHudList(NULL) {}
	~CHud();	// destructor, frees allocated memory

	// user messages
	int _cdecl MsgFunc_Damage( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_GameMode( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_Logo( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_ResetHUD( const char *pszName,  int iSize, void *pbuf );
	int _cdecl MsgFunc_InitHUD( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_ViewMode( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_SetFOV(const char *pszName,  int iSize, void *pbuf);
	int _cdecl MsgFunc_Concuss( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_RainData( const char *pszName, int iSize, void *pbuf ); 
	int _cdecl MsgFunc_HUDColor(const char *pszName,  int iSize, void *pbuf);
	int _cdecl MsgFunc_SetBody( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_SetSkin( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_Particle( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_KillPart( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_KillDecals( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_WeaponAnim( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_MusicFade( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_Weapons( const char *pszName, int iSize, void *pbuf );
	int  _cdecl MsgFunc_StudioDecal( const char *pszName, int iSize, void *pbuf );
	int  _cdecl MsgFunc_SetupBones( const char *pszName, int iSize, void *pbuf );
		
	// Screen information
	SCREENINFO m_scrinfo;

	byte m_iWeaponBits[MAX_WEAPON_BYTES];
	int m_fPlayerDead;
	int m_iIntermission;

	// sprite indexes
	int m_HUD_number_0;
	
	// error sprite
	int m_HUD_error;
	HSPRITE m_hHudError;
	
	void AddHudElem( CHudBase *p );
	float GetSensitivity() { return m_flMouseSensitivity; }
	BOOL HasWeapon( int weaponnum ) { return FBitSet( m_iWeaponBits[weaponnum >> 3], BIT( weaponnum & 7 )); }
	void AddWeapon( int weaponnum ) { SetBits( m_iWeaponBits[weaponnum >> 3], BIT( weaponnum & 7 )); }
};

extern CHud		gHUD;
extern hud_player_info_t	g_PlayerInfoList[MAX_PLAYERS+1];	// player info from the engine
extern extra_player_info_t	g_PlayerExtraInfo[MAX_PLAYERS+1];	// additional player info sent directly to the client dll
extern team_info_t		g_TeamInfo[MAX_TEAMS+1];