

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "soundent.h"
#include "decals.h"
#include "ent/func_break.h"

#define GERMAN_GIB_COUNT		4
#define HUMAN_GIB_COUNT		6
#define ALIEN_GIB_COUNT		4

 
BEGIN_DATADESC( CGib )
	DEFINE_FUNCTION( BounceGibTouch ),
	DEFINE_FUNCTION( StickyGibTouch ),
	DEFINE_FUNCTION( WaitTillLand ),
END_DATADESC()

// HACKHACK -- The gib velocity equations don't work
void CGib :: LimitVelocity( void )
{
	Vector vecNewVelocity = GetAbsVelocity();
	float length = vecNewVelocity.Length();

	// ceiling at 1500.  The gib velocity equation is not bounded properly.  Rather than tune it
	// in 3 separate places again, I'll just limit it here.
	if( length > 1500.0 )
	{
		vecNewVelocity = vecNewVelocity.Normalize() * 1500;	// This should really be sv_maxvelocity * 0.75 or something
		SetAbsVelocity( vecNewVelocity );
	}
}

//------------------------------------------------------------------------------
// A little piece of duplicated code
//------------------------------------------------------------------------------
void CGib :: AdjustVelocityBasedOnHealth( int nHealth, Vector &vecVelocity )
{
	if( nHealth > -50 )
	{
		vecVelocity *= 0.7f;
	}
	else if( nHealth > -200 )
	{
		vecVelocity *= 2;
	}
	else
	{
		vecVelocity *= 4;
	}
}

void CGib :: SpawnStickyGibs( entvars_t *pevVictim, Vector vecOrigin, int cGibs )
{
	int i;

	if ( g_Language == LANGUAGE_GERMAN )
	{
		// no sticky gibs in germany right now!
		return; 
	}

	for ( i = 0 ; i < cGibs ; i++ )
	{
		CGib *pGib = GetClassPtr( (CGib *)NULL );

		pGib->Spawn( "models/gibs/stickygib.mdl" );
		pGib->pev->body = RANDOM_LONG(0,2);

		if ( pevVictim )
		{
			pGib->SetLocalOrigin( Vector(
				vecOrigin.x + RANDOM_FLOAT( -3, 3 ),
				vecOrigin.y + RANDOM_FLOAT( -3, 3 ),
				vecOrigin.z + RANDOM_FLOAT( -3, 3 )
			));

			// make the gib fly away from the attack vector
			Vector vecNewVelocity = g_vecAttackDir * -1;

			// mix in some noise
			vecNewVelocity.x += RANDOM_FLOAT( -0.15f, 0.15f );
			vecNewVelocity.y += RANDOM_FLOAT( -0.15f, 0.15f );
			vecNewVelocity.z += RANDOM_FLOAT( -0.15f, 0.15f );

			vecNewVelocity *= 900;

			Vector vecAngVelocity( RANDOM_FLOAT( 250, 400 ), RANDOM_FLOAT( 250, 400 ), 0 );
			pGib->SetLocalAvelocity( vecAngVelocity );

 			pGib->AdjustVelocityBasedOnHealth( pevVictim->health, vecNewVelocity );
			pGib->SetAbsVelocity( vecNewVelocity );

			// copy owner's blood color
			pGib->m_bloodColor = (CBaseEntity::Instance(pevVictim))->BloodColor();

			pGib->pev->movetype = MOVETYPE_TOSS;
			pGib->pev->solid = SOLID_BBOX;
			UTIL_SetSize( pGib->pev, g_vecZero, g_vecZero );
			pGib->SetTouch( StickyGibTouch );
			pGib->SetThink( NULL );
		}

		pGib->LimitVelocity();
	}
}

void CGib :: SpawnHeadGib( entvars_t *pevVictim )
{
	CGib *pGib = GetClassPtr( (CGib *)NULL );

	if ( g_Language == LANGUAGE_GERMAN )
	{
		pGib->Spawn( "models/gibs/germangibs.mdl" );// throw one head
		pGib->pev->body = 0;
	}
	else
	{
		pGib->Spawn( "models/gibs/hgibs.mdl" );// throw one head
		pGib->pev->body = 0;
	}

	if ( pevVictim )
	{
		CBaseEntity *pVictim = CBaseEntity::Instance( pevVictim );
		Vector vecNewVelocity = pGib->GetAbsVelocity();

		pGib->SetLocalOrigin( pVictim->EyePosition() );
		
		edict_t *pentPlayer = FIND_CLIENT_IN_PVS( pGib->edict() );
		
		if ( RANDOM_LONG ( 0, 100 ) <= 5 && pentPlayer )
		{
			// 5% chance head will be thrown at player's face.
			CBaseEntity *pPlayer = CBaseEntity::Instance( pentPlayer );

			vecNewVelocity = (pPlayer->EyePosition() - pGib->GetAbsOrigin()).Normalize() * 300;
			vecNewVelocity.z += 100;
		}
		else
		{
			vecNewVelocity = Vector( RANDOM_FLOAT( -100, 100 ), RANDOM_FLOAT( -100, 100 ), RANDOM_FLOAT( 200, 300 ));
		}

		Vector vecNewAngularVelocity = pGib->GetLocalAvelocity();
		vecNewAngularVelocity.x = RANDOM_FLOAT( 100, 200 );
		vecNewAngularVelocity.y = RANDOM_FLOAT( 100, 300 );
		pGib->SetLocalAvelocity( vecNewAngularVelocity );

		// copy owner's blood color
		pGib->m_bloodColor = pVictim->BloodColor();

		pGib->AdjustVelocityBasedOnHealth( pevVictim->health, vecNewVelocity );
		pGib->SetAbsVelocity( vecNewVelocity );
	}

	pGib->LimitVelocity();
}

void CGib :: SpawnRandomGibs( entvars_t *pevVictim, int cGibs, int human )
{
	int cSplat;

	for ( cSplat = 0 ; cSplat < cGibs ; cSplat++ )
	{
		CGib *pGib = GetClassPtr( (CGib *)NULL );

		if ( g_Language == LANGUAGE_GERMAN )
		{
			pGib->Spawn( "models/gibs/germangibs.mdl" );
			pGib->pev->body = RANDOM_LONG(0,GERMAN_GIB_COUNT-1);
		}
		else
		{
			if ( human )
			{
				// human pieces
				pGib->Spawn( "models/gibs/hgibs.mdl" );
				pGib->pev->body = RANDOM_LONG(1,HUMAN_GIB_COUNT-1);// start at one to avoid throwing random amounts of skulls (0th gib)
			}
			else
			{
				// aliens
				pGib->Spawn( "models/gibs/agibs.mdl" );
				pGib->pev->body = RANDOM_LONG(0,ALIEN_GIB_COUNT-1);
			}
		}

		if ( pevVictim )
		{
			Vector vecOrigin;

			// spawn the gib somewhere in the monster's bounding volume
			vecOrigin.x = pevVictim->absmin.x + pevVictim->size.x * (RANDOM_FLOAT( 0 , 1 ) );
			vecOrigin.y = pevVictim->absmin.y + pevVictim->size.y * (RANDOM_FLOAT( 0 , 1 ) );
			vecOrigin.z = pevVictim->absmin.z + pevVictim->size.z * (RANDOM_FLOAT( 0 , 1 ) ) + 1;	// absmin.z is in the floor because the engine subtracts 1 to enlarge the box
			pGib->SetAbsOrigin( vecOrigin );

			// make the gib fly away from the attack vector
			Vector vecNewVelocity = g_vecAttackDir * -1;

			// mix in some noise
			vecNewVelocity.x += RANDOM_FLOAT ( -0.25, 0.25 );
			vecNewVelocity.y += RANDOM_FLOAT ( -0.25, 0.25 );
			vecNewVelocity.z += RANDOM_FLOAT ( -0.25, 0.25 );

			vecNewVelocity *= RANDOM_FLOAT ( 300, 400 );
			pGib->SetAbsVelocity( vecNewVelocity );

			Vector vecNewAngularVelocity = pGib->GetLocalAvelocity();
			vecNewAngularVelocity.x = RANDOM_FLOAT( 100, 200 );
			vecNewAngularVelocity.y = RANDOM_FLOAT( 100, 300 );
			pGib->SetLocalAvelocity( vecNewAngularVelocity );
 
			// copy owner's blood color
			pGib->m_bloodColor = (CBaseEntity::Instance(pevVictim))->BloodColor();

			pGib->AdjustVelocityBasedOnHealth( pevVictim->health, vecNewVelocity );			

			pGib->pev->solid = SOLID_BBOX;
			UTIL_SetSize( pGib->pev, g_vecZero, g_vecZero );
		}
		pGib->LimitVelocity();
	}
}


BOOL CBaseMonster :: HasHumanGibs( void )
{
	int myClass = Classify();

	if ( myClass == CLASS_HUMAN_MILITARY ||
		 myClass == CLASS_PLAYER_ALLY	||
		 myClass == CLASS_HUMAN_PASSIVE  ||
		 myClass == CLASS_PLAYER )

		 return TRUE;

	return FALSE;
}


BOOL CBaseMonster :: HasAlienGibs( void )
{
	int myClass = Classify();

	if ( myClass == CLASS_ALIEN_MILITARY ||
		 myClass == CLASS_ALIEN_MONSTER	||
		 myClass == CLASS_ALIEN_PASSIVE  ||
		 myClass == CLASS_INSECT  ||
		 myClass == CLASS_ALIEN_PREDATOR  ||
		 myClass == CLASS_ALIEN_PREY )

		 return TRUE;

	return FALSE;
}



//=========================================================
// WaitTillLand - in order to emit their meaty scent from
// the proper location, gibs should wait until they stop 
// bouncing to emit their scent. That's what this function
// does.
//=========================================================
void CGib :: WaitTillLand ( void )
{
	if( !IsInWorld( ))
	{
		UTIL_Remove( this );
		return;
	}

	if( GetAbsVelocity() == g_vecZero )
	{
		SetThink( SUB_StartFadeOut );
		SetNextThink( m_lifeTime );

		// If you bleed, you stink!
		if( m_bloodColor != DONT_BLEED )
		{
			// ok, start stinkin!
			CSoundEnt::InsertSound ( bits_SOUND_MEAT, GetAbsOrigin(), 384, 25 );
		}
	}
	else
	{
		// wait and check again in another half second.
		SetNextThink( 0.5 );
	}
}

//
// Gib bounces on the ground or wall, sponges some blood down, too!
//
void CGib :: BounceGibTouch( CBaseEntity *pOther )
{
	Vector vecSpot;
	TraceResult tr;

	if( pev->flags & FL_ONGROUND )
	{
		SetAbsVelocity( GetAbsVelocity() * 0.9f );
		Vector angles = GetLocalAngles();
		angles.x = 0;
		angles.z = 0;
		SetLocalAngles( angles );

		Vector angVel = GetLocalAvelocity();
		angVel.x = 0;
		angVel.z = 0;
		SetLocalAvelocity( angVel );	// g-cont. half-life rules: stay yaw rotation active
		pev->friction = 0.5;	// ... and enable Xash feature: friction
	}
	else
	{
		if( g_Language != LANGUAGE_GERMAN && m_cBloodDecals > 0 && m_bloodColor != DONT_BLEED )
		{
			vecSpot = GetAbsOrigin() + Vector ( 0 , 0 , 8 );//move up a bit, and trace down.
			UTIL_TraceLine( vecSpot, vecSpot + Vector( 0, 0, -24 ),  ignore_monsters, edict(), &tr );

			UTIL_BloodDecalTrace( &tr, m_bloodColor );

			m_cBloodDecals--; 
		}

		if( m_material != matNone && RANDOM_LONG( 0, 2 ) == 0 )
		{
			float volume;
			float zvel = fabs( GetAbsVelocity().z );
		
			volume = 0.8 * Q_min( 1.0, zvel / 450.0f);

			CBreakable::MaterialSoundRandom( edict(), (Materials)m_material, volume );
		}
	}
}

//
// Sticky gib puts blood on the wall and stays put. 
//
void CGib :: StickyGibTouch( CBaseEntity *pOther )
{
	TraceResult tr;
	
	SetThink( SUB_Remove );
	SetNextThink( 60 );

	if( pOther == g_pWorld || pOther->pev->movetype == MOVETYPE_PUSH || pOther->IsPushable() || pOther->IsRigidBody() )
	{
		UTIL_TraceLine ( GetAbsOrigin(), GetAbsOrigin() + GetAbsVelocity() * 32, ignore_monsters, edict(), &tr );
		UTIL_BloodDecalTrace( &tr, m_bloodColor );

		Vector vecForward = tr.vecPlaneNormal * -1;
		SetLocalAngles( UTIL_VecToAngles( vecForward ));
		SetLocalVelocity( g_vecZero ); 
		SetLocalAvelocity( g_vecZero );
		pev->movetype = MOVETYPE_NONE;

		if( pOther != g_pWorld )
			SetParent( pOther );
	}
	else
	{
		SetNextThink( 0 );
		return;
	}
}

//
// Throw a chunk
//
void CGib :: Spawn( const char *szGibModel )
{
	pev->movetype = MOVETYPE_BOUNCE;
	pev->friction = 0.55; // deading the bounce a bit
	
	// sometimes an entity inherits the edict from a former piece of glass,
	// and will spawn using the same render FX or rendermode! bad!
	pev->renderamt = 255;
	pev->rendermode = kRenderNormal;
	pev->renderfx = kRenderFxNone;
	pev->solid = SOLID_SLIDEBOX; // hopefully this will fix the VELOCITY TOO LOW crap
	pev->classname = MAKE_STRING( "gib" );

	SET_MODEL( edict(), szGibModel );
	UTIL_SetSize( pev, g_vecZero, g_vecZero );

	SetNextThink( 4 );
	m_lifeTime = GIBS_LIFE;
	SetThink( WaitTillLand );
	SetTouch( BounceGibTouch );

	m_material = matNone;
	m_cBloodDecals = 5; // how many blood decals this gib can place (1 per bounce until none remain). 
}
