//========= Copyright © 1996-2002, Valve LLC, All rights reserved. ============
//
// Purpose: тест 2.5д персонажей
//
// $NoKeywords: $
//=============================================================================

//=========================================================
// spritemonster test
//=========================================================

#include "monsters/sprite_monster.h"


//=========================================================
// Monster's Anim Events Go Here
//=========================================================



LINK_ENTITY_TO_CLASS( monster_sprite, CBaseSpriteMonster );



BEGIN_DATADESC( CBaseSpriteMonster )
        DEFINE_FIELD(currentAlight,FIELD_INTEGER),
		DEFINE_FIELD(currentAnim,FIELD_INTEGER),
	    DEFINE_FUNCTION( MonsterThink ),
END_DATADESC()




//=========================================================
// Classify - indicates this monster's place in the 
// relationship table.
//=========================================================
int	CBaseSpriteMonster :: Classify ( void )
{
	return	CLASS_HUMAN_MILITARY;
}

//=========================================================
// SetYawSpeed - allows each sequence to have a different
// turn rate associated with it.
//=========================================================
void CBaseSpriteMonster :: SetYawSpeed ( void )
{
	int ys;

	switch ( m_Activity )
	{
	case ACT_IDLE:
	default:
		ys = 90;
	}

	pev->yaw_speed = ys;
}

//=========================================================
// HandleAnimEvent - catches the monster-specific messages
// that occur when tagged animation frames are played.
//=========================================================
void CBaseSpriteMonster :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	switch( pEvent->event )
	{
	case 0:
	default:
		CBaseMonster::HandleAnimEvent( pEvent );
		break;
	}
}

//=========================================================
// ISoundMask - player monster can't hear.
//=========================================================
int CBaseSpriteMonster :: ISoundMask ( void )
{
	return	NULL;
}

//=========================================================
// Spawn
//=========================================================
void CBaseSpriteMonster :: Spawn()
{
	Precache( );

	SET_MODEL( edict(), "sprites/soldier.spr" );
	UTIL_SetSize(pev, VEC_HULL_MIN, VEC_HULL_MAX);

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
	m_bloodColor		= BLOOD_COLOR_RED;
	pev->health			= 88;
	pev->scale=1.5;
	m_flFieldOfView		= 0.5;// indicates the width of this monster's forward view cone ( as a dotproduct result )
	m_MonsterState		= MONSTERSTATE_NONE;
	MonsterInit();

currentAnim=ANIM_IDLE;
currentAlight=FRONT;

currentAnim=ANIM_IDLE;
Report();
SetAnim(ANIM_IDLE);
SetThink( MonsterThink );

}

//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CBaseSpriteMonster :: Precache()
{
	InitAnims();
PRECACHE_MODEL( "sprites/soldier.spr" );
}	


int CBaseSpriteMonster::TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )
{

	SetAnim(ANIM_PAIN);
	return CBaseMonster::TakeDamage(pevInflictor,pevAttacker,flDamage,bitsDamageType);
}


void CBaseSpriteMonster:: Killed( entvars_t *pevAttacker, int iGib )
{
    SetAnim(ANIM_KILLED);
	pev->solid= SOLID_NOT;


		if( pev->frame>=anim[currentAnim].endFrame[currentAlight])
		{
SetThink(NULL);
pev->frame=anim[currentAnim].endFrame[currentAlight];
		}
		CBaseMonster::Killed(pevAttacker,iGib);

}


void CBaseSpriteMonster::Report(void)
{
ALERT(at_console,"my aligh %i\n",currentAlight);
ALERT(at_console,"my frame %i\n",pev->frame);
ALERT(at_console,"my anim %i\n",anim[currentAnim].startFrame[currentAlight]);
//ALERT(at_console,"my aligh\n");


}


void CBaseSpriteMonster::MonsterThink(void)
{

if(pev->health<=0 && pev->frame>anim[currentAnim].endFrame[currentAlight])
return;

	pev->frame+=1;

	if(pev->frame>anim[currentAnim].endFrame[currentAlight] && anim[currentAnim].loop )
			pev->frame=anim[currentAnim].startFrame[currentAlight];


	if(!anim[currentAnim].loop && pev->frame>=anim[currentAnim].endFrame[currentAlight] && currentAnim!=ANIM_KILLED)
	{
Msg("animation end\n");
	SetAnim(ANIM_IDLE);
	}


findRorate();

		SetNextThink( anim[currentAnim].animSpeed );



}





void CBaseSpriteMonster :: SetAnim ( int pNewAnim )
{

currentAnim=pNewAnim;
pev->frame=anim[currentAnim].startFrame[currentAlight];
	SetThink(MonsterThink );
}




void CBaseSpriteMonster::InitAnims(void)
{


		anim[ANIM_IDLE].startFrame[FRONT]=IDLE_FRONT_START;
		anim[ANIM_IDLE].startFrame[BACK]=IDLE_BACK_START;
	anim[ANIM_IDLE].startFrame[SIDE]=IDLE_SIDE_START;


		anim[ANIM_IDLE].endFrame[FRONT]=IDLE_FRONT_END;
	anim[ANIM_IDLE].endFrame[BACK]=IDLE_BACK_END;
	anim[ANIM_IDLE].endFrame[SIDE]=IDLE_SIDE_END;
	anim[ANIM_IDLE].loop=TRUE;
	anim[ANIM_IDLE].animSpeed=0.1;


////////////////////////////////////////////////
    anim[ANIM_PAIN].startFrame[FRONT]=PAIN_FRONT_START;
	anim[ANIM_PAIN].startFrame[BACK]=PAIN_BACK_START;
	anim[ANIM_PAIN].startFrame[SIDE]=PAIN_SIDE_START;

	anim[ANIM_PAIN].endFrame[FRONT]=PAIN_FRONT_END;
	anim[ANIM_PAIN].endFrame[BACK]=PAIN_BACK_END;
	anim[ANIM_PAIN].endFrame[SIDE]=PAIN_SIDE_END;
	anim[ANIM_PAIN].loop=FALSE;
	anim[ANIM_PAIN].animSpeed=4;



//////////////////////////////////////////////

    anim[ANIM_KILLED].startFrame[FRONT]=DEATH_FRONT_START;
	anim[ANIM_KILLED].startFrame[BACK]=DEATH_BACK_START;
	anim[ANIM_KILLED].startFrame[SIDE]=DEATH_SIDE_START;

	anim[ANIM_KILLED].endFrame[FRONT]=DEATH_FRONT_END;
	anim[ANIM_KILLED].endFrame[BACK]=DEATH_BACK_END;
	anim[ANIM_KILLED].endFrame[SIDE]=DEATH_SIDE_END;
     anim[ANIM_KILLED].loop=FALSE;
	anim[ANIM_KILLED].animSpeed=0.1;
 ///////////////////////////////////////
    anim[ANIM_WALK].startFrame[FRONT]=WALK_FRONT_START;
    anim[ANIM_WALK].startFrame[BACK]=WALK_BACK_START;
	anim[ANIM_WALK].startFrame[SIDE]=WALK_SIDE_START;

	anim[ANIM_WALK].endFrame[FRONT]=WALK_FRONT_END;
	anim[ANIM_WALK].endFrame[BACK]=WALK_BACK_END;
	anim[ANIM_WALK].endFrame[SIDE]=WALK_SIDE_END;
anim[ANIM_WALK].loop=TRUE;
	anim[ANIM_WALK].animSpeed=0.1;
	///////////////////////////////////////

}



void CBaseSpriteMonster::InitAnims(spriteAnim_t panim,float speed,BOOL loop)
{
}

void CBaseSpriteMonster::findRorate(void)
{

	CBaseEntity	*pPlayer = UTIL_PlayerByIndex(1);

	if (!pPlayer)
		return;

	int orgframe = 0;

	float bepis = UTIL_VecToYaw(pPlayer->pev->origin - pev->origin);
	if (bepis > 180) bepis -= 360;
	if (bepis < -180) bepis += 360;
	//ALERT(at_console, "DOT %f\n", bepis);

	//8 SIDED
	if (bepis >= -22 && bepis <= 22)
	{
		orgframe = 0; //FORWARD
//	ALERT(at_console, "FORWARD\n");
	currentAlight=FRONT;
	}


	if (bepis >= 23 && bepis <= 67)
		orgframe = 1;

	if (bepis >= 68 && bepis <= 112)
	{
		orgframe = 2; //LEFT SIDE
	//	ALERT(at_console, "LEFT SIDE\n");
		currentAlight=SIDE;
	}

	if (bepis >= 113 && bepis <= 157)
		orgframe = 3;

	if (bepis >= 158 || bepis <= -157)
	{
		orgframe = 4; //BACKWARD
//	ALERT(at_console, " BACKWARD\n");
	currentAlight=BACK;
	}

	if (bepis >= -156 && bepis <= -113)
		orgframe = 5;

	if (bepis >= -112 && bepis <= -68)
	{
		orgframe = 6; //RIGHT SIDE
//	ALERT(at_console, " RIGHT SIDE\n");
	currentAlight=SIDE;
	}

	if (bepis >= -67 && bepis <= -23)
		orgframe = 7;

	float	yaw = pev->angles.y;
/*	int	angleframe = (int)(Q_rintt((yaw) / 360 * 8)) & 7;

	int finalframe = orgframe - angleframe;
	if (finalframe > 7) finalframe -= 8;
	if (finalframe < 0) finalframe += 8;

	ANIM_ANGLE_GROUP = abs(finalframe);//pev->frame = abs(finalframe);
	*/
}








//=========================================================
// AI Schedules Specific to this monster
//=========================================================
